@@ -19,3 +19,10 @@ def max_or_default(iterable, default) :
         return max(iterable)
     except ValueError :
         return default
+
+def as_numpy_scalar(x) :
+    """
+    Returns a :class:`numpy` scalar.
+    """
+    return np.array(x).dtype.type(x)
+
