@@ -87,6 +87,16 @@ class TestMVPolyInit(unittest.TestCase) :
                 self.assertTrue(p == r, "equality")
                 self.assertTrue(p.dtype == r.dtype, "dtype")
 
+    def test_default_dtype(self) :
+        def_dtype = np.double
+        for C in classes :
+            x, y = C.monomials(2)
+            self.assertTrue(x.dtype == def_dtype, "monomial")
+            p = C.zero()
+            self.assertTrue(p.dtype == def_dtype, "zero")
+            p = C.one()
+            self.assertTrue(p.dtype == def_dtype, "one")
+
 
 class TestMVPolyAsType(unittest.TestCase) :
 
@@ -136,6 +146,56 @@ class TestMVPolyAsClass(unittest.TestCase) :
                 self.assertTrue(p.dtype == r.dtype, "dtype")
 
 
+class TestMVPolyDtypeInherit(unittest.TestCase) :
+
+    @classmethod
+    def dtype_setup(self, C) :
+        x, y = C.monomials(2, dtype = int)
+        pi = (x + 2*y - 4)**3
+        pf = pi.astype(float)
+        return [pi, pf]
+
+    def test_dtype_inherit_add(self) :
+        for C in classes :
+            pi, pf = self.dtype_setup(C)
+            for r in [pi+pf, pf+pi, pf+pf, 1+pf, pf+1, 
+                      1.0+pf, pf+0.1, 1.0+pi, pi+1.0] :
+                self.assertTrue(r.dtype == float, 
+                                "%s %s add" % (C.__name__, repr(r.dtype)))
+            for r in [pi+pi, pi+1, 1+pi] :
+                self.assertTrue(r.dtype == int, 
+                                "%s %s add" % (C.__name__, repr(r.dtype)))
+
+    def test_dtype_inherit_minus(self) :
+        for C in classes :
+            pi, pf = self.dtype_setup(C)
+            for r in [pi-pf, pf-pi, pf-pf, 1-pf, pf-1, 
+                      1.0-pf, pf-0.1, 1.0-pi, pi-1.0] :
+                self.assertTrue(r.dtype == float, 
+                                "%s %s minus" % (C.__name__, repr(r.dtype)))
+            for r in [pi-pi, pi-1, 1-pi] :
+                self.assertTrue(r.dtype == int, 
+                                "%s %s minus" % (C.__name__, repr(r.dtype)))
+
+    def test_dtype_inherit_mul(self) :
+        for C in classes :
+            pi, pf = self.dtype_setup(C)
+            for r in [pi*pf, pf*pi, pf*pf, 1*pf, pf*1, 
+                      1.0*pf, pf*0.1, 1.0*pi, pi*1.0] :
+                self.assertTrue(r.dtype == float, 
+                                "%s %s mul" % (C.__name__, repr(r.dtype)))
+            for r in [pi*pi, pi*1, 1*pi] :
+                self.assertTrue(r.dtype == int, 
+                                "%s %s muk" % (C.__name__, repr(r.dtype)))
+
+    def test_dtype_inherit_scalar(self) :
+        for C in classes :
+            x, = C.monomials(1, dtype=np.int32)
+            v = np.int16(99)
+            froo = x.__rmul__(v)
+            self.assertTrue(froo.dtype == np.int32)
+
+
 class TestMVPolyGetitem(unittest.TestCase) :
 
     def test_getitem_numargs_zero(self) :
@@ -362,6 +422,14 @@ class TestMVPolyCompose(unittest.TestCase) :
 
 class TestMVPolyDiff(unittest.TestCase) :
 
+    def test_diff_0d(self) :
+        for C in classes :
+            p = 3 * C.one(dtype = int)
+            exp = 0
+            obt = p.diff(1)
+            self.assertTrue(exp == obt,
+                            "bad 1D-derivative\n%s\n%s" % (obt.coef, repr(exp)))
+
     def test_diff_1d(self) :
         for C in classes :
             x, = C.monomials(1)
@@ -383,20 +451,30 @@ class TestMVPolyDiff(unittest.TestCase) :
             obt = p.diff(0, 1)
             self.assertTrue(exp == obt,
                             "bad y-derivative\n%s\n%s" % (repr(obt), repr(exp)))
-
+            
     def test_diff_dtype(self) :
         for C in classes :
             for dt in (np.int32, np.float64) :
-                x, y = C.monomials(2, dtype=dt)
-                p = 2*x**2 + 3*y**3
+                p = C.zero(dtype = dt)
+                p[2, 0] = 4
+                p[0, 2] = 3
                 exp = dt
                 obt = p.diff(2,1).dtype
                 self.assertTrue(exp == obt, 
-                                "bad datatype %s %s" % (repr(obt), repr(exp)))
+                                "%s bad datatype %s %s" % (C, repr(obt), repr(exp)))
 
 
 class TestMVPolyInt(unittest.TestCase) :
 
+    def test_diff_0d(self) :
+        for C in classes :
+            x, = C.monomials(1, dtype = np.double)
+            p = 3 * C.one(dtype = np.double)
+            exp = 3*x
+            obt = p.int(1)
+            self.assertTrue(exp == obt,
+                            "bad 0D-integral\n%s\n%s" % (obt.coef, repr(exp)))
+
     def test_int_1d(self) :
         for C in classes :
             x, = C.monomials(1, dtype = np.double)
